/********************************************************************
Java TCP Client Applet demo program.
********************************************************************/


import java.applet.*;	// include for all applets
import java.awt.*;		// include for graphics
import java.awt.event.ActionListener;  // include to listen for events(buttons)
import java.awt.event.ActionEvent;  // include to perform action on events
import java.io.*;		// include to creat input and output streams
import java.net.*;      // include to use URLs and Sockets


/*-------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------*/

public class JavaClient extends Applet implements Runnable, ActionListener {
	static final String Message = "Java Client Demo"; //Message displayed at top of applet
	static final String CONNECT = "connect";		  //Defines action command of connect button
	static final String DISCONNECT = "disconnect";	  //Defines action command of disconnect button
    private Font font1 = new Font("Arial", Font.BOLD, 24);  //Defines font of message
    static int Port = 10000;	//Incoming Port on NetBurner Server
    URL root;					//stores IP address of NetBurner Server
	String ServerMessage;			//Stores streaming data from server
	Thread threadClient;		//Thread for the client to run in
	Image img;					//Will store the NetBurner logo for the applet
	InputStream istm;			//input stream for the incoming TCP connection from the Server
	InputStreamReader istmr;	//converts byte stream to a char stream
	BufferedReader data;		//Buffers char stream to allow lines to be read as strings
	Socket so;					//Socket connecting to the host server
	String RawData;				//Data string from server before converted to TimeMessage
	TextArea textArea;			//text area to display data from NetBurner Server
	Button con;					//button used to connect to server
	Button dis;					//button used to disconnect from server
	boolean connected = false;	//variable will define if user wishes to connect to server


/*----------------------------------------------------------------------------------------
OpenBackSocket() creates a new socket which will have an input data stream from the server
----------------------------------------------------------------------------------------*/

	void OpenBackSocket() {
	    try{
	   	   if (so!=null)		//if new socket to create is not null close it
		      so.close();
	       so = new Socket(root.getHost(),Port);	//create socket with server IP and port addresses
		   istm = so.getInputStream();				//initialize input stream of bytes
		   istmr = new InputStreamReader(istm);		//converts input stream to chars
		   data = new BufferedReader(istmr);		//buffers input stream for reading as a string
	    }
	    catch (Exception e) {		//if error creating socket make socket and input stream null
		    so=null;
		    istm=null;
		    System.out.println("Got exception :"+e+"In open socket");
		}
	}

/*--------------------------------------------------------------------------------------------
init() is called when the applet is first created
--------------------------------------------------------------------------------------------*/
	public void init() {
		//initializing components
		con  = new Button("Connect");  //initialize connect button and related actions to be enabled
		con.setActionCommand(CONNECT);
		con.addActionListener(this);
		con.setEnabled(true);
		dis  = new Button("Disonnect");	//initialize disconnect button and related actions to be diabled
		dis.setActionCommand(DISCONNECT);
		dis.addActionListener(this);
		dis.setEnabled(false);
		img = getImage( getDocumentBase(), "NetBurner-clear-small.gif" ); //load NetBurner logo to image
		textArea = new TextArea("Disconnected From NetBurner Server");  //create textArea for server data
    	textArea.setEditable(false);
    	textArea.setFocusable(false);
    	Panel p1 = new Panel();		//two panels created for layout of components
    	Panel p2 = new Panel();

    	//layout of components
    	setLayout(new BorderLayout());       // main layout
    	p1.setLayout( new BorderLayout() );  // layout for textarea
		p2.setLayout( new FlowLayout() );	 // layout for buttons
		add("South", p1);					 // put textArea at bottom of applet
		p1.add(p2, BorderLayout.NORTH);		 // put buttons above text area
		p2.add(con);						 // add objects to the layout
		p2.add(dis);
		p1.add(textArea);
	}


/*--------------------------------------------------------------------------------------------
paint() is called when the screen is repainted
--------------------------------------------------------------------------------------------*/
	public void paint(Graphics g) {
		g.drawImage( img, 0, 0, this );		//draw NetBurner logo
		g.setFont(font1);
		g.drawString(Message, 180, 45);		//draw message to right of logo
		g.drawString("Server IP: " + root.getHost() + " Port: " + Port, 25, 90);//draws servers IP and port
		textArea.setText(ServerMessage);  // Puts data from server into the textArea
	}

/*--------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------*/
	public void update(Graphics g) {  // update used to buffer repaint() to minimize flicker
		paint(g);
	}

/*--------------------------------------------------------------------------------------------
start() is called after init() to start the applet.
This method will create a thread for the applet to run in.
--------------------------------------------------------------------------------------------*/
	public void start() {
		if (threadClient == null) {
	    	threadClient = new Thread(this);
	   	    threadClient.start();
		}
	}

/*--------------------------------------------------------------------------------------------
stop() is called when applet is closed and will close open socket and stop the thread
--------------------------------------------------------------------------------------------*/
	public void stop() {
		threadClient = null;
		try{
		   so.close();
	    }
	   	catch (Exception e) {;}
	}

/*--------------------------------------------------------------------------------------------
run() is called after start() and contains the main TCP functionality of the applet.
The applet will remain in run() until the applet is terminated
--------------------------------------------------------------------------------------------*/
	public void run(){
		Thread myThread = Thread.currentThread();
        while (threadClient == myThread) {
			repaint();			//repaint applet and block thread
			if(connected){			 //if connect button enabled
				while(so==null) {	 //if socket is not opened then open socket
					if(root==null)
						root=getDocumentBase();
					OpenBackSocket();
				}
				if (istm!=null) {	 //if recieving streaming data from server
					try {
						RawData = data.readLine();		//read the input stream
						ServerMessage = RawData.replaceAll("zz", "\r\n");  //format input
					}
					catch (Exception e) {;}
				}
			}
			if(!connected){	     	//if disconnect button enabled
				try{
					so.close();		//close the socket
					so = null;		//asign null value to socket
					ServerMessage = "Disconnected From NetBurner Server";
				}
				catch (Exception e) {;}
			}
			try {
				Thread.sleep(50);	// sleep thread to slow loop
			}
			catch (InterruptedException e) { ; }
		}
	}

/*--------------------------------------------------------------------------------------------
actionPerformed() is called when either button on applet is selected
--------------------------------------------------------------------------------------------*/
	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand() == CONNECT) {	//if connect selected
			con.setEnabled(false);				//disable connect button
			dis.setEnabled(true);				//enable disconnect button
			connected = true;					//user wishes to connect to server
		}
		if (e.getActionCommand() == DISCONNECT) {
			con.setEnabled(true);
			dis.setEnabled(false);
			connected = false;
		}
	}
}